@@ -65,7 +65,7 @@ mod array_impl;
 
 pub mod messageitem;
 
-pub use self::msgarg::{Arg, FixedArray, Get, DictKey, Append, RefArg, AppendAll, ReadAll, ArgAll, cast, cast_mut};
+pub use self::msgarg::{Arg, FixedArray, Get, DictKey, Append, RefArg, AppendAll, ReadAll, ArgAll, cast, cast_mut, prop_cast};
 pub use self::array_impl::{Array, Dict};
 pub use self::variantstruct_impl::Variant;
 
@@ -74,6 +74,7 @@ use crate::{ffi, Message, Signature, Path};
 use std::ffi::{CStr, CString};
 use std::os::raw::{c_void, c_int};
 use std::os::unix::io::{RawFd, AsRawFd, FromRawFd, IntoRawFd};
+use std::collections::VecDeque;
 
 fn check(f: &str, i: u32) { if i == 0 { panic!("D-Bus error: '{}' failed", f) }}
 
@@ -261,7 +262,7 @@ impl<'a> Iter<'a> {
             ArgType::UInt64 => Box::new(self.get::<u64>().unwrap()),
             ArgType::Double => Box::new(self.get::<f64>().unwrap()),
             ArgType::UnixFd => Box::new(self.get::<OwnedFd>().unwrap()),
-            ArgType::Struct => Box::new(self.recurse(ArgType::Struct).unwrap().collect::<Vec<_>>()),
+            ArgType::Struct => Box::new(self.recurse(ArgType::Struct).unwrap().collect::<VecDeque<_>>()),
             ArgType::ObjectPath => Box::new(self.get::<Path>().unwrap().into_static()),
             ArgType::Signature => Box::new(self.get::<Signature>().unwrap().into_static()),
         })
@@ -273,7 +274,7 @@ impl<'a> Iter<'a> {
             let c = ffi::dbus_message_iter_get_signature(&mut self.0);
             assert!(c != ptr::null_mut());
             let cc = CStr::from_ptr(c);
-            let r = Signature::new(cc.to_bytes());
+            let r = Signature::new(std::str::from_utf8(cc.to_bytes()).unwrap());
             ffi::dbus_free(c as *mut c_void);
             r.unwrap()
         }
@@ -438,6 +439,11 @@ impl ArgType {
         ALL_ARG_TYPES.iter().skip_while(|a| a.0 != self).next().unwrap().1
     }
 
+    /// Returns a Vec of all possible argtypes.
+    pub fn all() -> Vec<Self> {
+        ALL_ARG_TYPES.iter().map(|x| x.0).collect()
+    }
+
     /// Converts an i32 to an ArgType (or an error).
     pub fn from_i32(i: i32) -> Result<ArgType, String> {
         for &(a, _) in &ALL_ARG_TYPES {
