@@ -1,9 +1,10 @@
 #![allow(dead_code)]
 
-use crate::{Signature, arg::TypeMismatchError};
+use crate::{Signature, arg::TypeMismatchError, arg::Variant};
 use std::{fmt, any};
 use std::sync::Arc;
-use std::rc::Rc;
+// use std::rc::Rc;
+use std::collections::HashMap;
 
 use super::{Iter, IterAppend, ArgType};
 
@@ -55,7 +56,7 @@ pub trait ReadAll: Sized {
 
 
 /// Object safe version of Arg + Append + Get.
-pub trait RefArg: fmt::Debug {
+pub trait RefArg: fmt::Debug + Send + Sync {
     /// The corresponding D-Bus argument type code.
     fn arg_type(&self) -> ArgType;
     /// The corresponding D-Bus type signature for this type.
@@ -64,12 +65,20 @@ pub trait RefArg: fmt::Debug {
     fn append(&self, _: &mut IterAppend);
     /// Transforms this argument to Any (which can be downcasted to read the current value).
     ///
-    /// Note: The internal representation of complex types (Array, Dict, Struct) is unstable
+    /// If the type is an array, and the element type is byte, numeric, boolean, or string, then the
+    /// representation is guaranteed to be a Vec<T>. Arrays of structs, dicts, and so on are not
+    /// safe to use with this.
+    ///
+    /// Note: The internal representation of complex types (Dict, Struct) is unstable
     /// and as_any should not be relied upon for these types. Use as_iter instead.
     fn as_any(&self) -> &dyn any::Any where Self: 'static;
     /// Transforms this argument to Any (which can be downcasted to read the current value).
     ///
-    /// Note: The internal representation of complex types (Array, Dict, Struct) is unstable
+    /// If the type is an array, and the element type is byte, numeric, boolean, or string, then the
+    /// representation is guaranteed to be a Vec<T>. Arrays of structs, dicts, and so on are not
+    /// safe to use with this.
+    ///
+    /// Note: The internal representation of complex types (Dict, Struct) is unstable
     /// and as_any should not be relied upon for these types. Use as_iter instead.
     ///
     /// # Panic
@@ -99,15 +108,22 @@ pub trait RefArg: fmt::Debug {
     /// Try to read the argument as an iterator.
     ///
     /// Works for: Array/Dict, Struct, Variant.
+    /// For Dicts, keys and values are interleaved.
     #[inline]
     fn as_iter<'a>(&'a self) -> Option<Box<dyn Iterator<Item=&'a dyn RefArg> + 'a>> { None }
+    /// Try to read the inner of an argument, as another argument, specifying an index.
+    ///
+    /// Works for: Variant, Array, Struct, Dict.
+    /// For Dicts, even indices gets a key, odd indices gets a value.
+    #[inline]
+    fn as_static_inner(&self, _index: usize) -> Option<&(dyn RefArg + 'static)> where Self: 'static { None }
     /// Deep clone of the RefArg, causing the result to be 'static.
     ///
     /// Usable as an escape hatch in case of lifetime problems with RefArg.
     ///
     /// In case of complex types (Array, Dict, Struct), the clone is not guaranteed
     /// to have the same internal representation as the original.
-    fn box_clone(&self) -> Box<dyn RefArg + 'static> { unimplemented!() /* Needed for backwards comp */ }
+    fn box_clone(&self) -> Box<dyn RefArg + 'static>;
 }
 
 impl<'a> Get<'a> for Box<dyn RefArg> {
@@ -126,6 +142,13 @@ pub fn cast<'a, T: 'static>(a: &'a (dyn RefArg + 'static)) -> Option<&'a T> { a.
 #[inline]
 pub fn cast_mut<'a, T: 'static>(a: &'a mut (dyn RefArg + 'static)) -> Option<&'a mut T> { a.as_any_mut().downcast_mut() }
 
+/// Descend into a hashmap returned by e g "Properties::get_all" to retrieve the value of a property.
+///
+/// Shortcut for get + cast. Returns None both if the property does not exist, or if it was of a different type.
+pub fn prop_cast<'a, T: 'static>(map: &'a HashMap<String, Variant<Box<dyn RefArg>>>, key: &str) -> Option<&'a T> {
+    map.get(key).and_then(|v| cast(&v.0))
+}
+
 /// If a type implements this trait, it means the size and alignment is the same
 /// as in D-Bus. This means that you can quickly append and get slices of this type.
 ///
@@ -169,6 +192,8 @@ impl<'a, T: RefArg + ?Sized> RefArg for &'a T {
     #[inline]
     fn as_iter<'b>(&'b self) -> Option<Box<dyn Iterator<Item=&'b dyn RefArg> + 'b>> { (&**self).as_iter() }
     #[inline]
+    fn as_static_inner(&self, index: usize) -> Option<&(dyn RefArg + 'static)> where Self: 'static { (&**self).as_static_inner(index) }
+    #[inline]
     fn box_clone(&self) -> Box<dyn RefArg + 'static> { (&**self).box_clone() }
 }
 
@@ -199,6 +224,8 @@ impl<T: RefArg + ?Sized> RefArg for $t<T> {
     #[inline]
     fn as_iter<'a>(&'a self) -> Option<Box<dyn Iterator<Item=&'a dyn RefArg> + 'a>> { (&**self).as_iter() }
     #[inline]
+    fn as_static_inner(&self, index: usize) -> Option<&(dyn RefArg + 'static)> where Self: 'static { (&**self).as_static_inner(index) }
+    #[inline]
     fn box_clone(&self) -> Box<dyn RefArg + 'static> { (&**self).box_clone() }
 }
 impl<T: DictKey> DictKey for $t<T> {}
@@ -219,7 +246,7 @@ impl<T: Append> Append for Box<T> {
 }
 
 deref_impl!(Box, self, &mut **self );
-deref_impl!(Rc, self, Rc::get_mut(self).unwrap());
+// deref_impl!(Rc, self, Rc::get_mut(self).unwrap());
 deref_impl!(Arc, self, Arc::get_mut(self).unwrap());
 
 macro_rules! argall_impl {
@@ -408,4 +435,61 @@ mod test {
             }
         }
     }
+
+    #[test]
+    fn cast_vecs() {
+        let c = Channel::get_private(BusType::Session).unwrap();
+
+        let m = Message::new_method_call(c.unique_name().unwrap(), "/hello", "com.example.hello", "Hello").unwrap();
+        macro_rules! append_array {
+            ($m:expr, $t:ty) => {
+                $m.append1(Variant(&Array::<&$t, _>::new(&vec![Default::default()])));
+            };
+        }
+        let m = append_array!(m, bool);
+        let m = append_array!(m, u8);
+        let m = append_array!(m, u16);
+        let m = append_array!(m, i16);
+        let m = append_array!(m, u32);
+        let m = append_array!(m, i32);
+        let m = append_array!(m, f64);
+        let m = append_array!(m, String);
+        c.send(m).unwrap();
+        loop {
+            if let Some(m) = c.blocking_pop_message(std::time::Duration::from_millis(1000)).unwrap() {
+                if m.msg_type() != MessageType::MethodCall {
+                    continue;
+                }
+                let mut i = m.iter_init();
+                let mut i2 = m.iter_init();
+
+                macro_rules! check_array {
+                    ($t:ty) => {
+                        let array: Variant<Box<dyn RefArg>> = i.read().unwrap();
+                        assert_eq!(
+                            cast::<Vec<$t>>(&(array.0)),
+                            Some(&vec![Default::default()]),
+                            "a variant containing an array of of {0} should be castable to a Vec<{0}>",
+                            std::any::type_name::<$t>()
+                        );
+                        let refarg = i2.get_refarg().unwrap();
+                        println!("refarg {:?}", refarg);
+                        let st_inner = refarg.as_static_inner(0).unwrap();
+                        println!("st_inner {:?}", st_inner);
+                        i2.next();
+                        assert_eq!(cast::<Vec<$t>>(st_inner), Some(&vec![Default::default()]));
+                    };
+                }
+                check_array!(bool);
+                check_array!(u8);
+                check_array!(u16);
+                check_array!(i16);
+                check_array!(u32);
+                check_array!(i32);
+                check_array!(f64);
+                check_array!(String);
+                break;
+            }
+        }
+    }
 }
