@@ -298,6 +298,11 @@ impl Channel {
         }
     }
 
+    /// Gets whether the output message buffer is non-empty
+    pub fn has_messages_to_send(&self) -> bool {
+        unsafe { ffi::dbus_connection_has_messages_to_send(self.conn()) == 1 }
+    }
+
     /// Removes a message from the incoming queue, or returns None if the queue is empty.
     ///
     /// Use "read_write" first, so that messages are put into the incoming queue.
@@ -395,6 +400,14 @@ impl Sender for std::cell::RefCell<Vec<Message>> {
     }
 }
 
+/// Use in case you don't want the send the message, but just collect it instead.
+impl Sender for std::sync::Mutex<Vec<Message>> {
+    fn send(&self, msg: Message) -> Result<u32, ()> {
+        self.lock().unwrap().push(msg);
+        Ok(0)
+    }
+}
+
 /// Token used to identify a callback in the MatchingReceiver trait
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub struct Token(pub usize);
