@@ -1,7 +1,7 @@
 //! Contains structs and traits closely related to D-Bus messages.
 
 use std::{fmt, ptr};
-use super::{ffi, Error, libc, to_c_str, c_str_to_slice, init_dbus};
+use super::{ffi, Error, libc, init_dbus};
 use crate::strings::{BusName, Path, Interface, Member, ErrorName};
 use std::ffi::CStr;
 
@@ -42,7 +42,7 @@ impl Message {
         init_dbus();
         let (d, p, i, m) = (destination.into(), path.into(), iface.into(), method.into());
         let ptr = unsafe {
-            ffi::dbus_message_new_method_call(d.as_ref().as_ptr(), p.as_ref().as_ptr(), i.as_ref().as_ptr(), m.as_ref().as_ptr())
+            ffi::dbus_message_new_method_call(d.as_ptr(), p.as_ptr(), i.as_ptr(), m.as_ptr())
         };
         if ptr.is_null() { Err("D-Bus error: dbus_message_new_method_call failed".into()) }
         else { Ok(Message { msg: ptr}) }
@@ -52,8 +52,8 @@ impl Message {
     pub fn method_call(destination: &BusName, path: &Path, iface: &Interface, name: &Member) -> Message {
         init_dbus();
         let ptr = unsafe {
-            ffi::dbus_message_new_method_call(destination.as_ref().as_ptr(), path.as_ref().as_ptr(),
-                iface.as_ref().as_ptr(), name.as_ref().as_ptr())
+            ffi::dbus_message_new_method_call(destination.as_ptr(), path.as_ptr(),
+                iface.as_ptr(), name.as_ptr())
         };
         if ptr.is_null() { panic!("D-Bus error: dbus_message_new_method_call failed") }
         Message { msg: ptr}
@@ -63,14 +63,13 @@ impl Message {
     pub fn call_with_args<'d, 'p, 'i, 'm, A, D, P, I, M>(destination: D, path: P, iface: I, method: M, args: A) -> Message
     where D: Into<BusName<'d>>, P: Into<Path<'p>>, I: Into<Interface<'i>>, M: Into<Member<'m>>, A: AppendAll {
         let mut msg = Message::method_call(&destination.into(), &path.into(), &iface.into(), &method.into());
-        args.append(&mut IterAppend::new(&mut msg));
+        msg.append_all(args);
         msg
     }
 
-
     /// Creates a new signal message.
     pub fn new_signal<P, I, M>(path: P, iface: I, name: M) -> Result<Message, String>
-    where P: Into<Vec<u8>>, I: Into<Vec<u8>>, M: Into<Vec<u8>> {
+    where P: Into<String>, I: Into<String>, M: Into<String> {
         init_dbus();
 
         let p = Path::new(path)?;
@@ -78,7 +77,7 @@ impl Message {
         let m = Member::new(name)?;
 
         let ptr = unsafe {
-            ffi::dbus_message_new_signal(p.as_ref().as_ptr(), i.as_ref().as_ptr(), m.as_ref().as_ptr())
+            ffi::dbus_message_new_signal(p.as_ptr(), i.as_ptr(), m.as_ptr())
         };
         if ptr.is_null() { Err("D-Bus error: dbus_message_new_signal failed".into()) }
         else { Ok(Message { msg: ptr}) }
@@ -88,7 +87,7 @@ impl Message {
     pub fn signal(path: &Path, iface: &Interface, name: &Member) -> Message {
         init_dbus();
         let ptr = unsafe {
-            ffi::dbus_message_new_signal(path.as_ref().as_ptr(), iface.as_ref().as_ptr(), name.as_ref().as_ptr())
+            ffi::dbus_message_new_signal(path.as_ptr(), iface.as_ptr(), name.as_ptr())
         };
         if ptr.is_null() { panic!("D-Bus error: dbus_message_new_signal failed") }
         Message { msg: ptr}
@@ -107,17 +106,18 @@ impl Message {
         Message {msg: ptr}
     }
 
-    /// The old way to create a new error reply
-    #[deprecated]
-    pub fn new_error(m: &Message, error_name: &str, error_message: &str) -> Option<Message> {
-        let (en, em) = (to_c_str(error_name), to_c_str(error_message));
-        let ptr = unsafe { ffi::dbus_message_new_error(m.msg, en.as_ptr(), em.as_ptr()) };
-        if ptr.is_null() { None } else { Some(Message { msg: ptr} ) }
+    /// Creates a reply for a method call message.
+    ///
+    /// Panics if called for a message which is not a method call.
+    pub fn return_with_args<A: AppendAll>(&self, args: A) -> Message {
+        let mut m = self.method_return();
+        m.append_all(args);
+        m
     }
 
     /// Creates a new error reply
     pub fn error(&self, error_name: &ErrorName, error_message: &CStr) -> Message {
-        let ptr = unsafe { ffi::dbus_message_new_error(self.msg, error_name.as_ref().as_ptr(), error_message.as_ptr()) };
+        let ptr = unsafe { ffi::dbus_message_new_error(self.msg, error_name.as_ptr(), error_message.as_ptr()) };
         if ptr.is_null() { panic!("D-Bus error: dbus_message_new_error failed") }
         Message { msg: ptr}
     }
@@ -216,6 +216,12 @@ impl Message {
         self
     }
 
+    /// Appends arguments to a message.
+    pub fn append_all<A: AppendAll>(&mut self, a: A) {
+        let mut m = IterAppend::new(self);
+        a.append(&mut m);
+    }
+
     /// Gets the first argument from the message, if that argument is of type G1.
     /// Returns None if there are not enough arguments, or if types don't match.
     pub fn get1<'a, G1: Get<'a>>(&'a self) -> Option<G1> {
@@ -338,9 +344,10 @@ impl Message {
         }
     }
 
-    fn msg_internal_str<'a>(&'a self, c: *const libc::c_char) -> Option<&'a [u8]> {
-        if c.is_null() { None }
-        else { Some( unsafe { CStr::from_ptr(c) }.to_bytes_with_nul()) }
+    fn msg_internal_str<'a>(&'a self, c: *const libc::c_char) -> Option<&'a str> {
+        if c.is_null() { return None };
+        let cc = unsafe { CStr::from_ptr(c) };
+        std::str::from_utf8(cc.to_bytes_with_nul()).ok()
     }
 
     /// Gets the name of the connection that originated this message.
@@ -349,18 +356,6 @@ impl Message {
             .map(|s| unsafe { BusName::from_slice_unchecked(s) })
     }
 
-    /// Returns a tuple of (Message type, Path, Interface, Member) of the current message.
-    #[deprecated]
-    pub fn headers(&self) -> (MessageType, Option<String>, Option<String>, Option<String>) {
-        let p = unsafe { ffi::dbus_message_get_path(self.msg) };
-        let i = unsafe { ffi::dbus_message_get_interface(self.msg) };
-        let m = unsafe { ffi::dbus_message_get_member(self.msg) };
-        (self.msg_type(),
-         c_str_to_slice(&p).map(|s| s.to_string()),
-         c_str_to_slice(&i).map(|s| s.to_string()),
-         c_str_to_slice(&m).map(|s| s.to_string()))
-    }
-
     /// Gets the object path this Message is being sent to.
     pub fn path(&self) -> Option<Path> {
         self.msg_internal_str(unsafe { ffi::dbus_message_get_path(self.msg) })
