@@ -74,6 +74,7 @@ pub struct LocalConnection {
     filters: RefCell<Filters<LocalFilterCb>>,
     replies: RefCell<Replies<LocalRepliesCb>>,
     timeout_maker: Option<TimeoutMakerCb>,
+    waker: Option<WakerCb>,
 }
 
 /// A connection to D-Bus, async version, which is Send but not Sync.
@@ -82,6 +83,7 @@ pub struct Connection {
     filters: RefCell<Filters<FilterCb>>,
     replies: RefCell<Replies<RepliesCb>>,
     timeout_maker: Option<TimeoutMakerCb>,
+    waker: Option<WakerCb>,
 }
 
 /// A connection to D-Bus, Send + Sync + async version
@@ -90,6 +92,7 @@ pub struct SyncConnection {
     filters: Mutex<Filters<SyncFilterCb>>,
     replies: Mutex<Replies<SyncRepliesCb>>,
     timeout_maker: Option<TimeoutMakerCb>,
+    waker: Option<WakerCb>,
 }
 
 use stdintf::org_freedesktop_dbus::DBus;
@@ -109,6 +112,7 @@ impl From<Channel> for $c {
             replies: Default::default(),
             filters: Default::default(),
             timeout_maker: None,
+            waker: None,
         }
     }
 }
@@ -134,11 +138,25 @@ impl MatchingReceiver for $c {
 impl NonblockReply for $c {
     type F = $rcb;
     fn send_with_reply(&self, msg: Message, f: Self::F) -> Result<Token, ()> {
-        self.channel.send(msg).map(|x| {
-            let t = Token(x as usize);
-            self.replies_mut().insert(t, f);
-            t
-        })
+        let token = {
+            // We must hold the mutex from moment we send the message
+            // To moment we set a handler for the reply
+            // So reply can't arrive before we set handler
+            let mut replies = self.replies_mut();
+            self.channel.send(msg).map(|x| {
+                let t = Token(x as usize);
+                replies.insert(t, f);
+                t
+            })
+        };
+        if self.channel.has_messages_to_send() {
+            // Non-blocking send failed
+            // Wake up task that will send the message
+            if self.waker.as_ref().map(|wake| wake().is_err() ).unwrap_or(false) {
+                return Err(());
+            }
+        }
+        token
     }
     fn cancel_reply(&self, id: Token) -> Option<Self::F> { self.replies_mut().remove(&id) }
     fn make_f<G: FnOnce(Message, &Self) + Send + 'static>(g: G) -> Self::F { Box::new(g) }
@@ -146,6 +164,9 @@ impl NonblockReply for $c {
     fn set_timeout_maker(&mut self, f: Option<TimeoutMakerCb>) -> Option<TimeoutMakerCb> {
         mem::replace(&mut self.timeout_maker, f)
     }
+    fn set_waker(&mut self, f: Option<WakerCb>) -> Option<WakerCb> {
+        mem::replace(&mut self.waker, f)
+    }
 }
 
 
@@ -267,6 +288,9 @@ impl SyncConnection {
 /// Internal callback for the executor when a timeout needs to be made.
 pub type TimeoutMakerCb = fn(timeout: Instant) -> pin::Pin<Box<dyn Future<Output=()> + Send + Sync + 'static>>;
 
+/// Internal callback for the executor when we need wakeup a task
+pub type WakerCb = Box<dyn Fn() -> Result<(), ()> + Send + Sync +'static>;
+
 /// Internal helper trait for async method replies.
 pub trait NonblockReply {
     /// Callback type
@@ -281,6 +305,8 @@ pub trait NonblockReply {
     fn set_timeout_maker(&mut self, f: Option<TimeoutMakerCb>) -> Option<TimeoutMakerCb>;
     /// Get the internal timeout maker
     fn timeout_maker(&self) -> Option<TimeoutMakerCb>;
+    /// Set the wakeup call
+    fn set_waker(&mut self, f: Option<WakerCb>) -> Option<WakerCb>;
 }
 
 
@@ -357,8 +383,8 @@ impl MsgMatch {
     /// Note: If the receiving end is disconnected and a message is received,
     /// the message matching will end but not in a clean fashion. Call remove_match() to
     /// stop matching cleanly.
-    pub fn msg_stream(self) -> (Self, futures::channel::mpsc::UnboundedReceiver<Message>) {
-        let (sender, receiver) = futures::channel::mpsc::unbounded();
+    pub fn msg_stream(self) -> (Self, futures_channel::mpsc::UnboundedReceiver<Message>) {
+        let (sender, receiver) = futures_channel::mpsc::unbounded();
         (self.msg_cb(move |msg| {
             sender.unbounded_send(msg).is_ok()
         }), receiver)
@@ -380,8 +406,8 @@ impl MsgMatch {
     ///    async {}
     /// });
     /// ```
-    pub fn stream<R: ReadAll + Send + 'static>(self) -> (Self, futures::channel::mpsc::UnboundedReceiver<(Message, R)>) {
-        let (sender, receiver) = futures::channel::mpsc::unbounded();
+    pub fn stream<R: ReadAll + Send + 'static>(self) -> (Self, futures_channel::mpsc::UnboundedReceiver<(Message, R)>) {
+        let (sender, receiver) = futures_channel::mpsc::unbounded();
         (self.cb(move |msg, r| {
             sender.unbounded_send((msg, r)).is_ok()
         }), receiver)
@@ -423,7 +449,7 @@ struct MRAwait {
 }
 
 async fn method_call_await(mra: MRAwait) -> Result<Message, Error> {
-    use futures::future;
+    use futures_util::future;
     let MRAwait { mrouter, token, timeout, timeoutfn } = mra;
     if token.is_err() { return Err(Error::new_failed("Failed to send message")) };
     let timeout = if let Some(tfn) = timeoutfn { tfn(timeout) } else { Box::pin(future::pending()) };
@@ -461,7 +487,7 @@ where
         args.append(&mut IterAppend::new(&mut msg));
         let mra = self.method_call_setup(msg);
         let r = method_call_await(mra);
-        let r = futures::FutureExt::map(r, |r| -> Result<R, Error> { r.and_then(|rmsg| rmsg.read_all()) } );
+        let r = futures_util::FutureExt::map(r, |r| -> Result<R, Error> { r.and_then(|rmsg| rmsg.read_all()) } );
         MethodReply::new(r)
     }
 }
@@ -481,7 +507,7 @@ impl Future for MROuter {
         let r = mem::replace(&mut *inner, MRInner::Neither);
         if let MRInner::Ready(r) = r { task::Poll::Ready(r) }
         else {
-            mem::replace(&mut *inner, MRInner::Pending(ctx.waker().clone()));
+            *inner = MRInner::Pending(ctx.waker().clone());
             return task::Poll::Pending
         }
     }
